c*-*
c*-* Copyright (C) 2000 Massimo Marchi and Piero Procacci
c*-* Full copyright notice at http://www.chim.unifi.it/orac/copyright4.0.html
c*-* Contact for info M. Marchi, CEA,  Gif Sur Yvette 91191 (FRANCE) 
c*-* Email:marchi@villon.saclay.cea.fr
c*-* 
      SUBROUTINE mtsmd(mapnl,xp0,yp0,zp0,xpg,ypg,zpg,eta,xpcm,ypcm,zpcm)

************************************************************************
*                                                                      *
*     MTSMD is the driver of the MD run when multiple time scales      *
*     methods are used. The integration algorithm is r-RESPA           *
*     of order o(dt^3) (Tuckermann et al. JCP 97 1990 (1992))          *
*                                                                      *
*                                                                      *
*----------------------------------------------------------------------*
*                                                                      *
*     Written by Piero Procacci, CECAM-ENS Lyon 1995                   *
*     New constant pressure and temperature algorithms implemented     *
*     by Massimo Marchi                                                *
*                                                                      *
*______________________________________________________________________*
*                                                                      *
*  MTSMD externals:                                                    *
*     adjust_bonds analyse_voronoi anneal appbou asng_xrmsw            *  
*     bcnstp calc_avg_str calc_avg_xrms calc_gofr calc_xrms            *
*     change_coord_inner change_frame change_origin check_topology     *
*     chkewl collision comp_abmd_fdiss comp_dip comp_fcm               *
*     comp_forcep comp_molmass comp_neigh_vor comp_stress_conf         *
*     comp_stress_kinetic comp_thermos_energy comp_thermos_forces      *
*     comp_vcm comp_vel_labframe comp_voronoi copy_protl correc        *
*     correc_etap correc_exp_scale correc_matr correc_stress cov_thermo*
*     cself daxpy dcopy dscal dumprs                                   *
*     erfc_spline_init erf_corr_cutoff ferrf fft_pme_init              *
*     find_diss_mol find_igint_vor find_thermos fndgrp gen_abmd_kvect  *
*     gen_stress_tag_bnd get_prot_cnstr get_type_slv icopy             *
*     inicmp int_corr_erf_spline kinetic lc_index lc_list              *
*     matinv mts_forces mts_furier mts_intra_n0   mts_intra_n1         *
*     mts_plotp mts_plot_fragm mts_test plotc plotd                    *
*     plot_center prtacc prtat prtba prtbnd                            *
*     prtcn prtfrc prtit prtite prtpt prtsq                            *
*     ranvel rattle_correc rattle_correc_co rattle_verlet              *
*     rattle_verlet_co readrs setup_skin_shell set_const_co            *
*     set_tempp set_tempt starting_verlet timer trans_center           *
*     tr_inbox verlet verlet_free verlet_free_eta write_bends          *
*     write_bonds write_confc write_gofrp write_gofrw write_pot_bond   *
*     write_pot_nbond write_tors xerror zero zero0                     *
*     zero3x3 zeroa zero_gofr zero_voronoi                             *
* 								       *
************************************************************************


*======================= DECLARATIONS ==================================

      IMPLICIT none
      
      include 'parst.h'

      INTEGER ma,mb,mh,ms,mf,memp,t1,mbs,numpr
      PARAMETER (ma=tsites,mb=tsitep,mh=elsiz,ms=secsiz,memp=1
     &     ,mf=mb*4,t1=types*(types+1)/2)

      PARAMETER (mbs=2*mb, numpr=npm)

*----------------------- ARGUMENTS -------------------------------------

      INTEGER mapnl(*)
      REAL*8  xp0(*),yp0(*),zp0(*),xpg(*),ypg(*),zpg(*),eta(*),xpcm(*)
     &     ,ypcm(*),zpcm(*)

*-------------------- VARIABLES IN COMMONS -----------------------------

      INCLUDE 'cpropar.h'
      include 'fourier.h'
      include 'pme.h'
      include 'giofar.h'
      include 'lc_list.h'

      INCLUDE 'unit.h'
      
*-------------------- DEFINITION OF AN EXTERNAL FUNCTION ---------------

      EXTERNAL  near0
      LOGICAL near0

*-------------------- LOCAL VARIABLES ----------------------------------

      CHARACTER*80 errmsg
      CHARACTER*7  eora
      character*1  rshell,rshk

      INTEGER    l2
      INTEGER    kp,kt,hours,min,nato_slt,iter_avg
      REAL*8  uptot,upstot,ustot,ektot,E0,fact

      INTEGER ninner,nscal,ntot_fragm,fragm_1,fragm_2,ninn1,ninn0
      INTEGER i,j,nsstt,nscale,ncconf,tstep,nstep,mstep,iret,ka
      INTEGER ig,flag,il,im,in0,in1,naux
      LOGICAL elflag,linit,lfirst,lprint
     &     ,lreturn,lupdate,lfalse,lrject
      REAL*8  virsp_h,virs_h,virs_m,virp_h,ucns_h,ucns_l,ucns_m,ucos_h
     &     ,ucos_l,ucos_m,ucnsp_h,ucnsp_l,ucnsp_m,ucosp_h,ucosp_l
     &     ,ucosp_m,virsp_l,virs_l,virp_l,ucnp_h,ucnp_l,ucnp_m,ucop_h
     &     ,ucop_l,ucop_m,tl,tm,tn0,tn1,tm2,tm4,tl2,time2,tn12,tn02
     &     ,virsp_m,de_remove,fudgec,fstep,dnit,dip_tot,dtvi,vi(mb)
      REAl*8 urcs_h,urcs_l,urcs_m,urcp_h,urcp_l,urcp_m,urcsp_h,urcsp_l
     &     ,urcsp_m,virp_m,eer_h,eer_l,eer_m,stressr_h(3,3),stressr_l(3
     &     ,3),stressr_m(3,3),stressr_n1(3,3),stressr_n0(3,3)
     &     ,stressd_h(3,3),stressd_l(3,3),stressd_m(3,3),stressd_n1(3,3)
     &     ,stressd_n0(3,3),stressd_p(3,3),stressd(3,3),stressr(3,3)
     &     ,stress_conf(3,3),stress_kin(3,3),stress_tot(3,3),prt_m(3,3)
     &     ,prt_l(3,3),prt_h(3,3),prt_n1(3,3),prt_n0(3,3),st_m(3,3)
     &     ,gmgp(3,3),press_m,press_l,press_h,press_n1
     &     ,press_n0,press_conf,press_kin

      REAL*8  temp,fcpu,elapse,puconf,pucoul,puhyd,pubnd
     &     ,ubend,uptors,uitors,uconf,ucoul,ureal,urecp,ucek,pucek,tempt
     &     ,fsrtal,fsrtalc,tempr,temppr,tcm,rcm,elaps,fnstep,rms,unbond
     &     ,cnbond,unb14,cnb14,timeq,xl,xlcut,sftalp,fsbond,fsbend
     &     ,fsin14,gsbond,gsbend,purecp,vfcp,tfcp,elps,etime,etimeq
     &     ,ttime,ttimeq,uumb,ungrp,cngrp,upconf,upcoul,timef,timefq
     &     ,gcpu,temph,uceh,hpot,ubond,aux,uslvbon,uslvben,gcpu_hd
     &     ,gcpu_ld,gcpu_md,gcpu_1,gcpu_2,gcpu_3,theoric_speed_up,ucepr
     &     ,gcpu_u,gcpu_0,gsin14,dips(3),vol_gofr,qt(4)
      REAL*8  gr,pueng
      INTEGER navg
      PARAMETER (navg = 60)

      REAL*8  sumarray(navg),ssmarray(navg)
      REAL*8 sum_econf,sum_ecoul,sum_enbnd,sum_etotpot,sum_tote,sum_ucek
     &     ,sum_temp,sum_tempt,sum_tempr,sum_temppr,sum_temph,sum_rms
     &     ,sum_pecek,sum_pehyd,sum_peconf,sum_pecoul,sum_percip
     &     ,sum_enb14,sum_ebend,sum_ebond,sum_eitor,sum_eptor,sum_pnbd
     &     ,sum_pebnd,sum_pepot,sum_ptote,sum_gr,sum_epcoul,sum_epconf
     &     ,sum_presst,sum_press,sum_pressc,sum_pressk,sum_st(3,3)
     &     ,sum_co(3,3),sum_volume,sum_pv,sum_temppra,sum_eslvint
     &     ,sum_eebond,sum_eebend,sum_eeptors,sum_eeitors

      REAL*8 ssm_econf,ssm_ecoul,ssm_enbnd,ssm_etotpot,ssm_tote,ssm_ucek
     &     ,ssm_temp,ssm_tempt,ssm_tempr,ssm_temppr,ssm_temph,ssm_rms
     &     ,ssm_pecek,ssm_pehyd,ssm_peconf,ssm_pecoul,ssm_percip
     &     ,ssm_enb14,ssm_ebend,ssm_ebond,ssm_eitor,ssm_eptor,ssm_pnbd
     &     ,ssm_pebnd,ssm_pepot,ssm_ptote,ssm_gr,ssm_epcoul,ssm_epconf
     &     ,ssm_presst,ssm_press,ssm_pressc,ssm_pressk,ssm_co(3,3)
     &     ,ssm_st(3,3),ssm_volume,ssm_pv,ssm_temppra,ssm_eslvint
     &     ,ssm_eebond,ssm_eebend,ssm_eeptors,ssm_eeitors

      EQUIVALENCE      
     &     (sum_econf, sumarray(1)), (sum_ecoul, sumarray(2)),
     &     (sum_enbnd, sumarray(3)), (sum_etotpot, sumarray(4)),
     &     (sum_tote, sumarray(5)), (sum_ucek, sumarray(6)),
     &     (sum_temp, sumarray(7)), (sum_tempt, sumarray(8)),
     &     (sum_tempr, sumarray(9)), (sum_temppr, sumarray(10)),
     &     (sum_temph, sumarray(11)), (sum_rms, sumarray(12)),
     &     (sum_pecek, sumarray(13)), (sum_pehyd, sumarray(14)),
     &     (sum_peconf, sumarray(15)), (sum_pecoul, sumarray(16)),
     &     (sum_enb14, sumarray(17)), (sum_ebend, sumarray(18)),
     &     (sum_ebond, sumarray(19)), (sum_eitor, sumarray(20)),
     &     (sum_eptor, sumarray(21)), (sum_pnbd, sumarray(22)),
     &     (sum_pebnd, sumarray(23)), (sum_pepot, sumarray(24)),
     &     (sum_ptote, sumarray(25)), (sum_gr, sumarray(26)),
     &     (sum_epcoul , sumarray(27)), (sum_epconf, sumarray(28)),
     &     (sum_st(1,1), sumarray(29)), (sum_co(1,1), sumarray(38)),
     &     (sum_presst, sumarray(47)), (sum_press, sumarray(48)),
     &     (sum_pressc, sumarray(49)), (sum_pressk, sumarray(50)),
     &     (sum_volume, sumarray(51)), (sum_pv, sumarray(52)),
     &     (sum_temppra,sumarray(53)), (sum_eslvint,sumarray(54)),
     &     (sum_percip,sumarray(55)),  (sum_eebond, sumarray(56)),
     &     (sum_eebend, sumarray(57)),  (sum_eeptors, sumarray(58)),
     &     (sum_eeitors, sumarray(59))
      EQUIVALENCE      
     &     (ssm_econf, ssmarray(1)), (ssm_ecoul, ssmarray(2)),
     &     (ssm_enbnd, ssmarray(3)), (ssm_etotpot, ssmarray(4)),
     &     (ssm_tote, ssmarray(5)), (ssm_ucek, ssmarray(6)),
     &     (ssm_temp, ssmarray(7)), (ssm_tempt, ssmarray(8)),
     &     (ssm_tempr, ssmarray(9)), (ssm_temppr, ssmarray(10)),
     &     (ssm_temph, ssmarray(11)), (ssm_rms, ssmarray(12)),
     &     (ssm_pecek, ssmarray(13)), (ssm_pehyd, ssmarray(14)),
     &     (ssm_peconf, ssmarray(15)), (ssm_pecoul, ssmarray(16)),
     &     (ssm_enb14, ssmarray(17)), (ssm_ebend, ssmarray(18)),
     &     (ssm_ebond, ssmarray(19)), (ssm_eitor, ssmarray(20)),
     &     (ssm_eptor, ssmarray(21)), (ssm_pnbd, ssmarray(22)),
     &     (ssm_pebnd, ssmarray(23)), (ssm_pepot, ssmarray(24)),
     &     (ssm_ptote, ssmarray(25)), (ssm_gr, ssmarray(26)),
     &     (ssm_epcoul , ssmarray(27)), (ssm_epconf, ssmarray(28)),
     &     (ssm_st(1,1), ssmarray(29)),(ssm_co(1,1),ssmarray(38)),
     &     (ssm_presst, ssmarray(47)), (ssm_press, ssmarray(48)),
     &     (ssm_pressc, ssmarray(49)), (ssm_pressk, ssmarray(50)),
     &     (ssm_volume, ssmarray(51)), (ssm_pv, ssmarray(52)),
     &     (ssm_temppra,ssmarray(53)), (ssm_eslvint,ssmarray(54)),
     &     (ssm_percip,ssmarray(55)),  (ssm_eebond, ssmarray(56)),
     &     (ssm_eebend, ssmarray(57)),  (ssm_eeptors, ssmarray(58)),
     &     (ssm_eeitors, ssmarray(59))

*=======================================================================
*    Variables needed to compute the stress tensor and implement       =
*    constant-stress simulations                                       =
*=======================================================================

      REAL*8  pressc
      REAL*8  stresstk(3,3)

      REAL*8  temppra,energy,time_fs



*----------- LOCAL WORK ARRAYS FOR THE RUN -----------------------------

      INTEGER mrject,ngrp_old,nprot_old,nind(2) 
      INTEGER*2  indxi(2,indmax),indxj(2,indmax),indxk(2,indmax)

      INTEGER*2 nnlpp0(mpp),nnlpp(mpp),nnlpp1(mpp),nnlpp2(1)
     &     ,nnlppf(sitslu+1)
      REAL*8  max_dist(3)
      REAL*8  fpx_n0(mb),fpy_n0(mb),fpz_n0(mb),fcax_n0(npm),fcay_n0(npm)
     &     ,fcaz_n0(npm)
      REAL*8  fpx_n1(mb),fpy_n1(mb),fpz_n1(mb),fcax_n1(npm),fcay_n1(npm)
     &     ,fcaz_n1(npm)
      REAL*8  fpx_m(mb),fpy_m(mb),fpz_m(mb),fcax_m(npm),fcay_m(npm)
     &     ,fcaz_m(npm)
      REAL*8  fpx_l(mb),fpy_l(mb),fpz_l(mb),fcax_l(npm),fcay_l(npm)
     &     ,fcaz_l(npm)
      REAL*8  fpx_h(mb),fpy_h(mb),fpz_h(mb),fcax_h(npm),fcay_h(npm)
     &     ,fcaz_h(npm)
      REAL*8  fpx(mb),fpy(mb),fpz(mb),grad_max

      REAL*8  fth(3)
      INTEGER mapdn(2,mf),nmapdn(mf)
      REAL*8  vpx(mb),vpy(mb),vpz(mb),vpx1(mb),vpy1(mb),vpz1(mb)
     &     ,vcax(numpr),vcay(numpr),vcaz(numpr),vcbx(numpr),vcby(numpr)
     &     ,vcbz(numpr)
      REAL*8  etap(hoov),vh1(hoov),vco(3,3)

*--- DYNAM is a scratch common block: here to save storage; not passed
*    to any of the external

      COMMON /dynam/ fpx_h,fpy_h,fpz_h,fpx_l,fpy_l,fpz_l,fpx_m,fpy_m
     &     ,fpz_m,fpx_n0,fpy_n0,fpz_n0,fpx_n1,fpy_n1,fpz_n1,fcax_m
     &     ,fcay_m,fcaz_m,fcax_l,fcay_l,fcaz_l,fcax_h,fcay_h,fcaz_h
     &     ,fcax_n0,fcay_n0,fcaz_n0,fcax_n1,fcay_n1,fcaz_n1,vpx,vpy,vpz
     &     ,vpx1,vpy1,vpz1,etap,vh1,vcax,vcay,vcaz,vcbx,vcby,vcbz,vco
     &     ,mapdn,nmapdn,nnlpp0,nnlpp,nnlpp1

*     Phony forces and energies for neighbor list

      REAL*8  ucns_p,ucos_p,virs_p,virsp_p,ucnp_p,ucop_p,ucnsp_p,ucosp_p
     &     ,fpx_p(1),fpy_p(1),fpz_p(1),rcuth_save,rtolh_save,rcutl_save
     &     ,rtoll_save,conf_bnd_slt,coul_bnd_slt,conf_bnd_slv
     &     ,coul_bnd_slv,self_slt,self_slv,uslvtor,uslvitor,fscnstr_slt
     &     ,fscnstr_slv,conf_bnd_slt_n1,coul_bnd_slt_n1,conf_bnd_slv_n1
     &     ,coul_bnd_slv_n1
      
      LOGICAL dpress,dhoover,grflag,exist,mask(m1),h_skin,l_skin,m_skin
     &     ,n1_skin,n0_skin
      INTEGER dthree
      REAL*8 zz1(3,3),lzz,lzz1
      INTEGER igrn,krdf(maxint*g1),worka(m1,m10),type_slv(slvatm)
      INTEGER numatoms,ntype_slv,offset_slv,nbetab_slv,abmd_cryst_nvect
      INTEGER tag_bndg(m2)
      PARAMETER(nbetab_slv=90,abmd_cryst_nvect=10)
      INTEGER itype_slv(nbetab_slv)
      CHARACTER*1 betab_slv(nbetab_slv)
      REAL*8  dssp(mbs),coeffp(mbs)
      REAL*8  xpo(mb),ypo(mb),zpo(mb),coo(3,3),co2(3,3),oc2(3,3)
      REAL*8  xpa(mb),ypa(mb),zpa(mb),xpga(m11),ypga(m11),zpga(m11)
     &     ,xpcma(npm),ypcma(npm),zpcma(npm),lx(mb),ly(mb),lz(mb)
      REAL*8  work(mspline),wca(m1),whe(m1),wbc(m1),errca(npm),errhe(npm
     &     ),errbc(npm),erral(npm),drpca(m1),drpbc(m1),drphe(m1)
     &     ,drpal(m1),xp_avg(m1),yp_avg(m1),zp_avg(m1),tmass(numpr)
     &     ,tmassb(numpr),masspp(3),dssco(5),cnstco(2,5)
     &     ,abmd_cryst_vect(3,abmd_cryst_nvect),rtollo

      INTEGER cnstp(2,mbs),cnstpp,offset,abmd_dir,cnstpp_slv
     &     ,cnst_protl(2*m1),cnst_protp,count,tot_protl,tot_cnst
     &     ,nmol_slt,nmol_slv,natom_slt,natom_slv

      COMMON /rag2/ temp,fcpu,elapse,puconf,pucoul,puhyd,pubnd,ubend
     &     ,uptors,uitors,uconf,ucoul,ureal,urecp,ucek,pucek,tempt
     &     ,fsrtal,tempr,temppr,tcm,rcm,fnstep,rms,unbond,cnbond,unb14
     &     ,cnb14,timeq,xl,xlcut,sftalp,fsbond,fsbend,fsin14,gsbond
     &     ,gsbend,purecp,vfcp,tfcp,elps,etime,etimeq,ttime,ttimeq,uumb
     &     ,ungrp,cngrp,upconf,upcoul,timef,timefq,gcpu,temph,uceh,hpot
     &     ,ubond,aux,ucepr,temppra,dssp,coeffp,xpo,ypo,zpo,xpa,ypa,zpa
     &     ,xpga,ypga,zpga,xpcma,ypcma,zpcma,wca,whe,wbc,errca,errhe
     &     ,errbc,erral,drpca,drpbc,drphe,drpal,xp_avg,yp_avg,zp_avg
     &     ,tmass,tmassb,lx,ly,lz,work,krdf,cnstp,cnst_protl
     &     ,tag_bndg

      DATA eora/'E.O.R. '/

*==================== EXECUTABLE STATEMENTS ============================



      lfalse = .FALSE.
      IF(.NOT.clean) THEN
         WRITE(kprint,20900)
      END IF   
*===  Check if the dimension of the work array are sufficient 


      IF(mb.LT.ntap) THEN
         errmsg=' While running MD: PARAMETER MB dimensions the work'
     &        //' arrays in sufficiently. Abort. '
         CALL xerror(errmsg,80,1,2)
         STOP
      END IF

      if (lrespa.EQ.0) lrespa=1
      if (mrespa.EQ.0) mrespa=1
      if (n1respa.Eq.0)n1respa=1
   
*=======================================================================
*----- Initialize some stuff -------------------------------------------
*=======================================================================

*===  set few variable to zero

      lreturn=.false.
      lprint=.false.
      flag=nflag(1)
      IF(flag .EQ. 1) THEN
         mrject=0
         nrject=0
      END IF
      mrject=nrject*lrespa*mrespa
      time_fs=time*DFLOAT(maxstp+nrject)
      nscale=0
      ncconf=0
      mstep=0
      urcs_h=0.0D0
      urcs_l=0.0D0
      urcs_m=0.0D0
      urcp_h=0.0D0
      urcp_l=0.0D0
      urcp_m=0.0D0
      urcsp_h=0.0D0
      urcsp_l=0.0D0
      urcsp_m=0.0D0
      eer_h=0.0D0
      eer_l=0.0D0
      eer_m=0.0D0
      dip_tot=0.d0
      CALL zero3x3(stressr_h)
      CALL zero3x3(stressr_l)
      CALL zero3x3(stressr_m)
      CALL zero3x3(stressr_n1)
      CALL zero3x3(stressr_n0)
      CALL zero3x3(stressd_h)
      CALL zero3x3(stressd_l)
      CALL zero3x3(stressd_m)
      CALL zero3x3(stressd_n1)
      CALL zero3x3(stressd_n0)
      uumb=0.0D0
      conf_bnd_slv=0.0D0
      conf_bnd_slt=0.0D0
      tstep=0
      cnstpp=0
      fudgec=1.d0-fudge
      nsstt=0
      iret=0
      errmsg=' '
      nato_slt=ntap-nmol*nato_slv
      iter_avg=0
      IF(cpress) THEN
         DO i=1,3
            masspp(i)=masspr
         END DO
         IF(isostress) CALL set_const_co(co,dssco,cnstco)
      END IF
      CALL comp_molmass(nprot,protl,mass,tmass)

      IF(voronoi) THEN
         CALL find_igint_vor(ngrp,grppt,iret,errmsg)
         IF(iret .EQ. 1) CALL xerror(errmsg,80,1,2)
      END IF

      IF(abmd) THEN
         IF(fold) THEN
            IF(.NOT. abmd_unbias) THEN
               IF(rspset .LT. 0.0D0) THEN
                  abmd_dir=-1
               ELSE
                  abmd_dir=1
               END IF
               rspset=0.0D0
            END IF
         ELSE IF(dissociate .OR. associate) THEN
            IF(dissociate) THEN
               abmd_dir=1
            ELSE IF(associate) THEN
               abmd_dir=-1
            END IF
            rspset=0.0D0
            CALL find_diss_mol(diss_mol,diss_atoms,atoms_diss,mol_diss
     &           ,diss_list,nprot,protl)
         ELSE IF(abmd_cryst) THEN
            rspset=0.0D0
            abmd_dir=abmd_cryst_dir
            CALL gen_abmd_kvect(abmd_kvect,abmd_cryst_nvect
     &           ,abmd_cryst_vect)
         END IF
      END IF

*=======================================================================
*--- Initialize the velocities if at the beginning of the run        ---
*=======================================================================

      CALL zeroa(vpx,vpy,vpz,ntap,1)
      CALL zeroa(vpx1,vpy1,vpz1,ntap,1)
      CALL zeroa(vcax,vcay,vcaz,nprot,1)
      CALL zeroa(xp_avg,yp_avg,zp_avg,ntap,1)

      if(start_conf) then 
         max_dist(1) = distmax
         max_dist(2) = distmax
         max_dist(3) = distmax
      end if

      IF(nplot_fragm .GT. 0) THEN
         do i=1,nfragm
            IF ( fragm(1,i).gt.ntap.or.fragm(2,i).gt.ntap) THEN 
               WRITE(kprint,10260) i
10260          FORMAT(//  
     &              '**ERROR: fragments limits for',i5,' DEF_FRAGM',
     &              '(&READ_SOLUTE) exceeds protein atoms' )
               STOP
            END IF 
         end do
         ntot_fragm=0
         do i=1,nfragm
            fragm_1=fragm(1,i)
            fragm_2=fragm(2,i)
            ntot_fragm = ntot_fragm + fragm_2-fragm_1+1 
         end do    
      END IF

*=======================================================================
*----- Copy old CO and OC matrix to temporary arrays -------------------
*=======================================================================

      IF(change_cell) THEN
         CALL dcopy(9,co,1,co2,1)
         CALL dcopy(9,oc,1,oc2,1)
      END IF

*=======================================================================
*----- Read input from restart file if needed --------------------------
*=======================================================================

      kp=0
      kt=-1
      IF(flag.GT.0) THEN
         elflag=electr.AND.(.NOT.elinit)
         WRITE(kprint,70300)
         CALL readrs(kdump,nstep,temp,ntap,ngrp_old,nprot_old,xp0,yp0
     &        ,zp0,vpx,vpy,vpz,xpg,ypg,zpg,xpcma,ypcma,zpcma,vpx1,vpy1
     &        ,vpz1,vcax,vcay,vcaz,eta,etap,vh1,dthree,dhoover,sumarray
     &        ,ssmarray,navg,navg,co,oc,dpress,vco,zz1,lzz,lzz1,grflag
     &        ,krdf,igrn,maxint*g1,restart_cont)
         WRITE(kprint,70400)

         ninner=nstep*lrespa*mrespa
         ninn1=nstep*lrespa*mrespa*n1respa

         IF(flag .EQ. 1) THEN
            time_fs=time*DFLOAT(maxstp-nstep)
         END IF
      END IF

      IF(flag .EQ. 1 .AND. nstep .GE. maxstp ) THEN
         WRITE(errmsg,'(''In MTSMD: Step number in restart = '',
     &        i6,'' while in input = '',i6,'' Increase TIME.'')')
     &        nstep,maxstp
         CALL xerror(errmsg,80,1,2)
      END IF

*=======================================================================
*---  Write a Banner Page                                            ---
*=======================================================================

      WRITE(kprint,'(//)')
      WRITE(kprint,1200)
      WRITE(kprint,1300)
      WRITE(kprint,1100)
      WRITE(kprint,1100)
      WRITE(kprint,1300)
      WRITE(kprint,1200)
      WRITE(kprint,'(//)')

*=======================================================================
*---  Reset the averages when NFLAG = 0 or 2                         ---
*=======================================================================

      IF(flag.EQ.2. .OR. flag .EQ. 3 .OR.flag.EQ.0 
     &     .OR. flag .EQ. 4) THEN
          CALL zero(sumarray,ssmarray,navg,navg)
         nstep=0
         ninner=0
         ninn1=0
         ninn0=0
      END IF

      IF(anxrms .OR. gofr .OR. avg_str) CALL asng_xrmsw(ss_point,m1+1
     &     ,wca,whe,wbc,beta,mback,nbone)

      IF(anxrms_cell) THEN
         anprot=.TRUE.
         annpro=annpro+1
         anpoint(1,annpro)=1
         anpoint(2,annpro)=ntap
      END IF


      IF(gofr) CALL get_type_slv(nato_slt,nato_slv,beta,betab_slv
     &     ,nbetab_slv,ntype_slv,type_slv,itype_slv,offset_slv
     &     ,types_gofr,iret,errmsg)
      IF(iret .EQ. 1) CALL xerror(errmsg,80,1,2)

*=======================================================================
*---- Initialize G of Rs -----------------------------------------------
*=======================================================================

      IF(gofr) THEN
         CALL zero_gofr(maxint,krdf,ngrdon,offset_slv)
      END IF

*=======================================================================
*-------- Compute two vectors used by the constraint subroutine --------
*=======================================================================

      IF(lconstr .NE. 0) THEN
         IF(adjust_cnstr) THEN
            WRITE(kprint,14000) 
            CALL adjust_bonds(ss_index,ntap,lcnstr,lconstr
     &        ,xp0,yp0,zp0,potbo_cnst,M9,iret,errmsg)
            IF(iret .EQ. 2) CALL xerror(errmsg,80,1,20)
         END IF
         cnstpp=0
         CALL bcnstp(ss_index,lcnstr,lconstr,xp0,yp0,zp0,mass,
     &        dssp,coeffp,cnstp,mbs,cnstpp,cnstpp_slv,iret,errmsg)
         IF(iret .EQ. 1) CALL xerror(errmsg,80,1,2)
         IF(stretch_heavy) THEN
            CALL get_prot_cnstr(ntap,clsthl,nclsth,cnstp,cnstpp
     &           ,cnst_protl,cnst_protp,tot_protl,tot_cnst,mask,worka
     &           ,m1)
         ELSE
            CALL get_prot_cnstr(ntap,protl,nprot,cnstp,cnstpp,cnst_protl
     &           ,cnst_protp,tot_protl,tot_cnst,mask,worka,m1)
         END IF
      END IF

      CALL copy_protl(protl,protlb,nprot,nprotb)
      IF(coupl_atm .OR. coupl_grp) THEN
         CALL comp_molmass(nprot,protl,mass,tmassb)
         IF(lconstr .NE. 0) THEN
            nprot=nclsth
            CALL icopy(tot_protl,clsthl,1,protl,1)
         ELSE
            nprot=ntap
            count=0
            DO i=1,ntap
               protl(1+count)=1
               protl(2+count)=i
               count=count+2
            END DO
         END IF
         IF(nprot .GT. npm) THEN
            errmsg=
     &           'Number of molecules exceeds physical dimensions. '
     &           / /'Increase _N_PROT_MAX_ to '
            WRITE(errmsg(75:80),'(i6)') nprot
            CALL xerror(errmsg,80,1,2)            
         END IF
         CALL comp_molmass(nprot,protl,mass,tmass)
         CALL comp_nmol(ss_index,nprot,protl,nmol_slt,nmol_slv,natom_slt
     &        ,natom_slv)
      ELSE
         CALL comp_nmol(ss_index,nprot,protl,nmol_slt,nmol_slv,natom_slt
     &        ,natom_slv)
      END IF

*=======================================================================
*----- Choose on which shell to carry out the gmgp correction ----------
*=======================================================================

      IF(cpress) THEN
         CALL setup_skin_shell(coupl_grp,coupl_mol,h_skin,l_skin,m_skin
     &        ,n1_skin,n0_skin)
      END IF

*=======================================================================
*----- Calculate solute center of mass coordinates and velocities ------
*=======================================================================

      CALL inicmp(ss_index,xp0,yp0,zp0,xpcm,ypcm,zpcm,mass,nprot,protl)

*=======================================================================
*----- Change the cell according to input commands ---------------------
*=======================================================================

      IF(change_cell) THEN
         CALL change_frame(co,oc,-1,nprot,xpcm,ypcm,zpcm,xpcma,ypcma
     &        ,zpcma)
         CALL change_origin(1,nprot,protl,xp0,yp0,zp0,lx,ly,lz,xpcma
     &        ,ypcma,zpcma,co)

         CALL dcopy(9,co2,1,co,1)
         CALL dcopy(9,oc2,1,oc,1)

         CALL change_origin(-1,nprot,protl,xp0,yp0,zp0,lx,ly,lz,xpcma
     &        ,ypcma,zpcma,co)
         CALL change_origin(1,nprot,protl,xp0,yp0,zp0,lx,ly,lz,xpcma
     &        ,ypcma,zpcma,co)
         CALL change_frame(co,oc,1,nprot,xpcma,ypcma,zpcma,xpcm,ypcm
     &        ,zpcm)
      END IF

*=======================================================================
*---- Calculate group position  ----------------------------------------
*=======================================================================

      CALL appbou(xp0,yp0,zp0,xpg,ypg,zpg,pmass,ngrp,grppt)
      CALL comp_vcm(vpx,vpy,vpz,oc,nprot,protl,mass,tmass,vcax,vcay
     &        ,vcaz)
      
*=======================================================================
*-------- Find out the first and last group of each protein ------------
*=======================================================================

      CALL fndgrp(nprot,ngrp,protl,grppt,atomg,protg,groupp,atomp,npm
     &     ,iret,errmsg)
      IF(iret .EQ. 1) CALL xerror(errmsg,80,1,2)

      IF(nflag(1) .NE. 0 .AND. ngrp_old .NE. ngrp ) THEN
         WRITE(kprint,98000) ngrp_old,ngrp
      END IF
      IF(nflag(1) .NE. 0 .AND. nprot_old .NE. nprot ) THEN
         WRITE(kprint,99000) nprot_old,nprot
      END IF

*=======================================================================
*--- Initialize stress tag tables for bendings -------------------------
*=======================================================================

      CALL gen_stress_tag_bnd(lbend,3,lbndg,atomp,tag_bndg)

*=======================================================================
*----- If the system is at constant temperature initialize some --------
*----- variables and convert masses of the 3 thermostats ---------------
*=======================================================================

      IF(thermos) THEN
         CALL find_thermos(ntap,lconstr,natom_slt,natom_slv,nmol_slt
     &        ,nmol_slv,cnstpp_slv,nprot,cpress,isostress,ndf_thermos)
     &        
         CALL cov_thermos(slv_exist,slt_exist,qmass,ndf_thermos,t)
         IF(flag .EQ. 0) THEN
            CALL zero0(eta,neta)
         END IF
      END IF

*=======================================================================
*-------- Initialize velocities when starting without a restart --------
*=======================================================================

      IF(FLAG.EQ.0.AND.t.GT.1d-15) THEN
         LINIT=.TRUE.
         CALL ranvel(t,mass,ntap,vpx,vpy,vpz,xp0,yp0,zp0,linit)
         IF(cnstpp .NE. 0) THEN
            CALL rattle_correc(time,xp0,yp0,zp0,vpx,vpy
     &           ,vpz,ntap,cnstp,dssp,coeffp,cnstpp,mass
     &           ,dnit,cnst_protp,cnst_protl,mim_lim,iret,errmsg)
            IF(iret .EQ. 1) CALL xerror(errmsg,80,1,2)
         END IF
         CALL comp_vcm(vpx,vpy,vpz,oc,nprot,protl,mass,tmass,vcax,vcay
     &        ,vcaz)
         LINIT=.FALSE.

*=======================================================================
*----- Set to zero velocities of the barostat --------------------------
*=======================================================================

         IF(cpress) THEN
            CALL set_tempp(masspr,vco,temppra,0.0D0)
         END IF
         IF(thermos) THEN
            CALL set_tempt(neta,qmass,etap,temph,0.0D0)
         END IF
      END IF   

*=======================================================================
*--- Compute the volume of the system ----------------------------------
*=======================================================================

      CALL matinv(3,3,co,oc,volume)
      volume=volume*boxl**3
     
*========================================================================
*==== Calls init routine for conventional kspace Ewald or PME -----------
*========================================================================

      IF(clewld) THEN

         IF(pme) THEN
            numatoms=ntap
            CALL fft_pme_init(numatoms,nfft1,nfft2,nfft3,pme_order
     &           ,sizfftab,sizffwrk,siztheta,siz_Q,sizheap,sizstack
     &           ,bsp_mod1,bsp_mod2,bsp_mod3,fftable,ffwork)
            if ( siz_Q .GT. MAXT ) THEN
               write(kprint,78410)
               stop
            END IF
            rshk=shell_pme
            CALL erf_corr_cutoff(oc,delew,rkcut,nfft1,nfft2,nfft3)
            IF(erf_corr) THEN
               CALL int_corr_erf_spline(rlew,ruew,nbinew,alphal,rkcut
     &           ,erf_arr_corr,work)
            END IF
         ELSE
            aux=(rtolh+rcuth)**2
            CALL chkewl(oc,aux,alphal,rkcut,volume)
            rshk=shell_pme
            IF(erf_corr) THEN  
              CALL int_corr_erf_spline(rlew,ruew,nbinew,alphal,rkcut
     &           ,erf_arr_corr,work)
            END IF
         END IF

*=======================================================================
*----- Set up table for erfc spline ------------------------------------
*=======================================================================

         IF(erfc_spline) THEN
            aux=rcuth+rtolh
            aux=aux*aux
            CALL erfc_spline_init(aux,alphal,erfc_bin,mspline,erfc_arr
     &           ,work,iret,errmsg,rkcut,rcut_corr,erfc_spline_corr)
            IF(iret .EQ. 1) CALL xerror(errmsg,80,1,2)
         END IF
      END IF

*=======================================================================
*---- Calculate ewald self and the intramolecular terms. ---------------
*---- Only the contribution to the latter coming from  -----------------
*---- bond constraints is computed at this point -----------------------
*=======================================================================

      IF(clewld) THEN
         CALL cself(ss_index,ntap,alphal,rkcut,chrge,self_slt,self_slv)
         CALL ferrf(ss_index,alphal,chrge,1.0D0,xp0,yp0,zp0,0,lcnstr
     &        ,lconstr,fscnstr_slt,fscnstr_slv,fpx_p,fpy_p,fpz_p
     &        ,erf_corr,erf_arr_corr,delew,rlew)
      END IF

*=======================================================================
*---  set fake neighbor lists
*=======================================================================

      nnlppf(1)=ngrp
      DO j=1,ngrp
         nnlppf(1+j)=j
      end do

*=======================================================================
*---  Check if there are no error in the list of bonds, bendings -------
*---  and torsions -----------------------------------------------------
*=======================================================================

      IF(prttopl) THEN
         CALL check_topology('B',top_bonds,lbond,iret,errmsg)
         IF(iret .NE. 0) CALL xerror(errmsg,80,1,2)
         CALL check_topology('D',top_bendings,lbend,iret,errmsg)
         IF(iret .NE. 0) CALL xerror(errmsg,80,1,2)
         CALL check_topology('P',top_ptors,ltors,iret,errmsg)
         IF(iret .NE. 0) CALL xerror(errmsg,80,1,2)
         CALL check_topology('I',top_itors,litor,iret,errmsg)
         IF(iret .NE. 0) CALL xerror(errmsg,80,1,2)
      END IF


*=======================================================================
*----- Print titles for the run ----------------------------------------
*=======================================================================


      CALL prtite
      IF(prttpg) THEN
         WRITE(kprint,1000)
         WRITE(kprint,'(////)')
         IF(prtseq) CALL prtsq(nbun,mend,prsymb)
         IF(prtatl) CALL prtat(ss_index,ntap,beta,betb,nres,m1,prsymb
     &        ,chrge,mass)
         IF(prtbndl) CALL prtbnd(beta,nres(1,1),lstrtch,lstretch,potbo
     &        ,m9)
         IF(prtcnl) CALL prtcn(beta,nres(1,1),lcnstr,lconstr,potbo_cnst
     &        ,m9)
         IF(prtbal) CALL prtba(beta,nres(1,1),lbndg,lbend,potbe,m2)
         IF(prtptl) CALL prtpt(beta,nres(1,1),ltor,ltors,potto,m3)
         IF(prtitl) CALL prtit(beta,nres(1,1),litr,litor,potit,m4)
         WRITE(kprint,'(////77(''=''))')
      END IF

*======================================================================
*==== Write force field when required. Only the parameters used -------
*==== in the actual simulation will be printed ------------------------
*======================================================================

      IF(write_ff_pars) THEN
         WRITE(kprint,89000)
         WRITE(kprint,90000)
         CALL write_pot_bond(lstretch,2,lstrtch,m9,2,potbo,betb,1)
         WRITE(kprint,95000)
         CALL write_pot_bond(lconstr,2,lcnstr,m9,2,potbo_cnst,betb,-1)
         WRITE(kprint,94000)
         WRITE(kprint,91000)
         CALL write_pot_bond(lbend,3,lbndg,m2,4,potbe,betb,1)
         WRITE(kprint,94000)
         WRITE(kprint,92000)
         CALL write_pot_bond(ltors,4,ltor,m3,2,potto,betb,1)
         WRITE(kprint,94000)
         WRITE(kprint,93000)
         CALL write_pot_bond(litor,4,litr,m4,3,potit,betb,1)
         WRITE(kprint,94000)
         WRITE(kprint,96000)
         WRITE(kprint,97000)
         CALL write_pot_nbond(ecc6,ecc12,ecc146,ecc1412,nbtype,mass
     &        ,lj_fudgeb,betb,ntap)
         WRITE(kprint,94000)
         WRITE(kprint,89500)
      END IF

*======================================================================
*==== Write a Banner Before intermediate results ----------------------
*======================================================================

      WRITE(kprint,'(//)')
      WRITE(kprint,1200)
      WRITE(kprint,1300)
      WRITE(kprint,1110)
      WRITE(kprint,1110)
      WRITE(kprint,1300)
      WRITE(kprint,1200)
      WRITE(kprint,'(//)')

      IF(rneih.GT.0.d0) THEN
         lupdate=.true.
      else
         nnlpp0(1)=ngrp
         DO j=1,ngrp
            nnlpp0(1+j)=j
         end do
         lupdate=.false.
      END IF   

*=======================================================================
*--- Change frame to get xpa, ypa, zpa etc in box fractions ------------
*=======================================================================

      CALL change_frame(co,oc,-1,ntap,xp0,yp0,zp0,xpa,ypa,zpa)
      CALL change_frame(co,oc,-1,ngrp,xpg,ypg,zpg,xpga,ypga,zpga)
      CALL change_frame(co,oc,-1,nprot,xpcm,ypcm,zpcm,xpcma,ypcma,zpcma)

*=======================================================================
*--- Compute lx, ly, lz by subtracting the coordinates of the c.of m.---
*=======================================================================

      CALL change_origin(1,nprot,protl,xp0,yp0,zp0,lx,ly,lz,xpcma,ypcma
     &     ,zpcma,co)

*==== Phony call to forces: Computes only neighbor lists (OLD UPDATE)          

      CALL timer(vfcp,tfcp,elapse)
      gcpu=tfcp
      IF(lupdate.and.(.not.linked_cell)) THEN
*--      update shell h neighbor list
         CALL mts_forces('u',xpa,ypa,zpa,xpga,ypga,zpga,xpcma,ypcma
     &        ,zpcma,mapnl,mapdn,nmapdn,ucns_p,ucos_p,virs_p,virsp_p
     &        ,ucnp_p,ucop_p,ucnsp_p,ucosp_p,fpx_p,fpy_p,fpz_p,stressd_p
     &        ,nnlppf,nnlpp0)
      ELSE
         aux= rcuth+rtolh+rneih
         CALL lc_index(indmax,ncx,ncy,ncz,nind,indxi,indxj,indxk,aux,co)
         CALL timer(vfcp,tfcp,elapse)
         gcpu=-gcpu + tfcp
         write(kprint,15011) gcpu
         CALL timer(vfcp,tfcp,elapse)
         gcpu=tfcp
         CALL lc_list(ncx,ncy,ncz,nind,indxi,indxj,indxk,aux,co,xpga
     &        ,ypga,zpga,ngrp,nnlpp0,kprint)
      END IF   

      CALL timer(vfcp,tfcp,elapse)
      gcpu=-gcpu + tfcp
      gcpu_u=gcpu
      write(kprint,16011) gcpu

*=======================================================================
*---- Zeroes all forces ------------------------------------------------
*=======================================================================

      CALL zeroa(fpx_h,fpy_h,fpz_h,ntap,1)
      CALL zeroa(fpx_l,fpy_l,fpz_l,ntap,1)
      CALL zeroa(fpx_m,fpy_m,fpz_m,ntap,1)
      CALL zeroa(fpx_n1,fpy_n1,fpz_n1,ntap,1)
      CALL zeroa(fpx_n0,fpy_n0,fpz_n0,ntap,1)

      write(kprint,15000) 

*=======================================================================
*===== Computes all forces at time = 0 ================================= 
*=======================================================================

************************************************************************
***                        Direct SPACE                              ***
***        keep this order to compute nested neighbor lists          ***
************************************************************************

*=======================================================================
*==== H-contribution in direct space
*=======================================================================

      CALL timer(vfcp,tfcp,elapse)
      gcpu=tfcp

*---  CALLs mts_forslv,mts_fnbond,mts_forpw

      rshell='h'
      CALL mts_forces(rshell,xpa,ypa,zpa,xpga,ypga,zpga,xpcma,ypcma
     &     ,zpcma,mapnl,mapdn,nmapdn,ucns_h,ucos_h,virs_h,virsp_h,ucnp_h
     &     ,ucop_h,ucnsp_h,ucosp_h,fpx_h,fpy_h,fpz_h,stressd_h,nnlpp0
     &     ,nnlpp)
      CALL timer(vfcp,tfcp,elapse)
      gcpu_hd=-gcpu + tfcp
      gcpu_1 = gcpu_hd

*=======================================================================
*==== L-contribution in direct space (long-ranged) ---------------------
*=======================================================================

      CALL timer(vfcp,tfcp,elapse)
      gcpu=tfcp

      rshell='l'
      CALL mts_forces(rshell,xpa,ypa,zpa,xpga,ypga,zpga,xpcma,ypcma
     &     ,zpcma,mapnl,mapdn,nmapdn,ucns_l,ucos_l,virs_l,virsp_l,ucnp_l
     &     ,ucop_l,ucnsp_l,ucosp_l,fpx_l,fpy_l,fpz_l,stressd_l,nnlpp
     &     ,nnlpp1)
      CALL timer(vfcp,tfcp,elapse)
      gcpu_ld=-gcpu + tfcp
      gcpu_2 = gcpu_ld

*=======================================================================
*------- M-contribution in direct space (short-ranged) -----------------
*-------          and protein torsion ----------------------------------
*=======================================================================

      CALL timer(vfcp,tfcp,elapse)
      gcpu=tfcp
      rshell='m'
      CALL mts_forces(rshell,xpa,ypa,zpa,xpga,ypga,zpga,xpcma,ypcma
     &     ,zpcma,mapnl,mapdn,nmapdn,ucns_m,ucos_m,virs_m,virsp_m,ucnp_m
     &     ,ucop_m,ucnsp_m,ucosp_m,fpx_m,fpy_m,fpz_m,stressd_m,nnlpp1
     &     ,nnlpp2)

      IF(abmd .AND. (dissociate .OR. associate)) CALL
     &     comp_abmd_fdiss(abmd_dir,rspset,diss_list,spring,xpa,ypa,zpa
     &     ,co,fpx_m,fpy_m,fpz_m,uumb,gr)

      CALL timer(vfcp,tfcp,elapse)
      gcpu_md=-gcpu + tfcp
      gcpu_3 = gcpu_md

************************************************************************
***                    Reciprocal SPACE                              ***
***     order is reverted to avoid to compute twice pme corrections  ***
************************************************************************

*=======================================================================
*==== M-contribution in reciprocal Space -------------------------------
*=======================================================================

      IF(.NOT.clewld) goto 907 
      gcpu=0.d0
      rshell='m'
      IF(pme .AND. rshell .NE. rshk) goto 914
      CALL timer(vfcp,tfcp,elapse)
      gcpu=tfcp

*---  CALLs mts_furier, mts_furipp, mts_furipw 

      CALL mts_furier(xp0,yp0,zp0,xpa,ypa,zpa,xpcma,ypcma,zpcma,urcsp_m
     &     ,urcs_m,urcp_m,virsp_m,virs_m,virp_m,fpx_m,fpy_m,fpz_m,fsin14
     &     ,gsin14,fsbend,gsbend,fsbond,gsbond,fscnstr_slt,fscnstr_slv
     &     ,coul_bnd_slt,coul_bnd_slv,rshell,rshk,eer_m,stressr_m,fudgec
     &     ,tag_bndg)

      CALL timer(vfcp,tfcp,elapse)
      gcpu=-gcpu + tfcp
914   write(kprint,16003) gcpu,gcpu_md,gcpu+gcpu_md
      gcpu_1=gcpu+gcpu_md

*=======================================================================
*==== L-contribution in reciprocal Space -------------------------------
*=======================================================================

      gcpu=0.d0
      rshell='l'
      IF(pme .AND. rshell .NE. rshk) goto 915
      CALL timer(vfcp,tfcp,elapse)
      gcpu=tfcp

*--   subtract the less accurate k-forces

      CALL mts_furier(xp0,yp0,zp0,xpa,ypa,zpa,xpcma,ypcma,zpcma,urcsp_l
     &     ,urcs_l,urcp_l,virsp_l,virs_l,virp_l,fpx_l,fpy_l,fpz_l,fsin14
     &     ,gsin14,fsbend,gsbend,fsbond,gsbond,fscnstr_slt,fscnstr_slv
     &     ,coul_bnd_slt,coul_bnd_slv,rshell,rshk,eer_l,stressr_l,fudgec
     &     ,tag_bndg)

      CALL timer(vfcp,tfcp,elapse)
      gcpu=-gcpu + tfcp
915   write(kprint,16002) gcpu,gcpu_ld,gcpu+gcpu_ld
      gcpu_2=gcpu+gcpu_ld


*=======================================================================
*==== H-contribution in reciprocal Space -------------------------------
*=======================================================================

      gcpu=0.d0
      rshell='h'
      IF(pme .AND. (rshell .NE. rshk)) goto 916
      CALL timer(vfcp,tfcp,elapse)
      gcpu=tfcp

*--   subtract the less accurate k-forces

      CALL mts_furier(xp0,yp0,zp0,xpa,ypa,zpa,xpcma,ypcma,zpcma,urcsp_h
     &     ,urcs_h,urcp_h,virsp_h,virs_h,virp_h,fpx_h,fpy_h,fpz_h,fsin14
     &     ,gsin14,fsbend,gsbend,fsbond,gsbond,fscnstr_slt,fscnstr_slv
     &     ,coul_bnd_slt,coul_bnd_slv,rshell,rshk,eer_h,stressr_h,fudgec
     &     ,tag_bndg)

      CALL timer(vfcp,tfcp,elapse)

      gcpu=-gcpu + tfcp
916   write(kprint,16001) gcpu,gcpu_hd,gcpu+gcpu_hd
      gcpu_3=gcpu+gcpu_hd
907   theoric_speed_up=(gcpu_1+gcpu_2+gcpu_3)*mrespa*lrespa/(gcpu_1
     &     *mrespa*lrespa+gcpu_2*lrespa+gcpu_3)

*=======================================================================
*--- Compute force on the co matrix ------------------------------------
*=======================================================================

      IF(cpress) THEN
         CALL comp_stress_conf(stressd_m,stressr_m,prt_m,oc,volume,unitp
     &        ,press_m)
         CALL comp_stress_kinetic(vcax,vcay,vcaz,tmass,co,nprot,volume
     &        ,unitp,st_m,press_kin)
         CALL comp_forcep(prt_m,st_m,oc,volume,pext)
         CALL comp_stress_conf(stressd_l,stressr_l,prt_l,oc,volume,unitp
     &        ,press_l)
         CALL comp_stress_conf(stressd_h,stressr_h,prt_h,oc,volume,unitp
     &        ,press_h)
      END IF

*=======================================================================
*--- Print timing for spherical cutoff ---------------------------------
*=======================================================================

      IF(.NOT.clewld) THEN
         gcpu=0.d0
         write(kprint,16001) gcpu,gcpu_hd,gcpu_hd
         write(kprint,16002) gcpu,gcpu_ld,gcpu_ld
         write(kprint,16003) gcpu,gcpu_md,gcpu_md
         gcpu_1=gcpu+gcpu_md
         gcpu_2=gcpu+gcpu_ld
         gcpu_3=gcpu+gcpu_hd
      END IF   
      write(kprint,10067) theoric_speed_up

************************************************************************
***            FAST INTRAMOLECULAR COMPONENTS                        ***
************************************************************************

*=======================================================================
*---- Computes bonded forces of shell n1 -------------------------------
*=======================================================================

      CALL timer(vfcp,tfcp,elapse)
      gcpu=tfcp
      
      CALL mts_intra_n1(xp0,yp0,zp0,xpcma,ypcma,zpcma,fpx_n1,fpy_n1
     &     ,fpz_n1,fudge,lj_fudge,abmd_dir,puhyd,conf_bnd_slt_n1
     &     ,conf_bnd_slv_n1,coul_bnd_slt_n1,coul_bnd_slv_n1,unb14,cnb14
     &     ,ungrp,cngrp,uptors,uslvtor,stressd_n1,mapdn,nmapdn,uumb,gr)

      CALL timer(vfcp,tfcp,elapse)
      gcpu=-gcpu + tfcp
      write(kprint,14004) gcpu
      gcpu_0=gcpu

*=======================================================================
*---- Computes bonded forces of shell n0 -------------------------------
*=======================================================================

      CALL timer(vfcp,tfcp,elapse)
      gcpu=tfcp

      CALL mts_intra_n0(xp0,yp0,zp0,xpcma,ypcma,zpcma,fpx_n0,fpy_n0
     &     ,fpz_n0,ubond,uslvbon,ubend,uslvben,uitors,uslvitor
     &     ,stressd_n0,tag_bndg)


      CALL timer(vfcp,tfcp,elapse)

*=======================================================================
*---- Compute stress tensors from bonded interaction -------------------
*=======================================================================

      IF(cpress) THEN
         CALL comp_stress_conf(stressd_n1,stressr_n1,prt_n1,oc,volume
     &        ,unitp,press_n1)
         CALL comp_stress_conf(stressd_n0,stressr_n0,prt_n0,oc,volume
     &        ,unitp,press_n0)
      END IF

*=======================================================================
*--- Compute force on the c.of.m of each molecule and reduce the -------
*--- atomic force. Do so for all shell forces --------------------------
*=======================================================================

      CALL comp_fcm(nprot,protl,fpx_m,fpy_m,fpz_m,fcax_m,fcay_m,fcaz_m
     &     ,mass,tmass,oc)

      CALL comp_fcm(nprot,protl,fpx_l,fpy_l,fpz_l,fcax_l,fcay_l,fcaz_l
     &     ,mass,tmass,oc)

      CALL comp_fcm(nprot,protl,fpx_h,fpy_h,fpz_h,fcax_h,fcay_h,fcaz_h
     &     ,mass,tmass,oc)

      IF(coupl_grp) THEN
         CALL comp_fcm(nprot,protl,fpx_n1,fpy_n1,fpz_n1,fcax_n1,fcay_n1
     &        ,fcaz_n1,mass,tmass,oc)
         CALL comp_fcm(nprot,protl,fpx_n0,fpy_n0,fpz_n0,fcax_n0,fcay_n0
     &        ,fcaz_n0,mass,tmass,oc)
      END IF

*=======================================================================
*---- Print timing and simulation CPU time estimates -------------------
*=======================================================================
      
      gcpu=-gcpu + tfcp
      write(kprint,16004) gcpu
      theoric_speed_up=(gcpu+gcpu_0+gcpu_1+gcpu_2+gcpu_3)*n0respa
     &     *n1respa*mrespa*lrespa/(gcpu*n0respa*n1respa*mrespa*lrespa
     &     +gcpu_0*n1respa*mrespa*lrespa+gcpu_1*mrespa*lrespa+gcpu_2
     &     *lrespa+gcpu_3)
      write(kprint,10068) theoric_speed_up
      if(nupdte.gt.0) gcpu_u = gcpu_u /dfloat(nupdte)
      aux = (gcpu*n0respa*n1respa*lrespa*mrespa+gcpu_0*n1respa*lrespa
     &     *mrespa+gcpu_1*mrespa*lrespa+gcpu_2*lrespa + gcpu_3 + gcpu_u)
      gcpu_0 = aux/dfloat(lrespa*mrespa)
      gcpu_1 = aux / time
      aux = aux*(maxstp+(mrject/dfloat(lrespa*mrespa)))
      hours = int(aux/3600.d0)
      min = int((aux-hours*3600)/60.d0)
      write(kprint,80033) hours,min,gcpu_0,gcpu_1
      write(kprint,60030) 

*=======================================================================
*===== MD loop starts here =============================================
*=======================================================================

      CALL timer(vfcp,tfcp,elapse)
      gcpu=tfcp
      elaps=elapse
      lfirst=.true.
      time2=time*0.5D0

      if(energy_then_die) go to 1010
100   CONTINUE

      nstep=nstep+1
      mstep=mstep+1

*=======================================================================
*---- Start 4 time steps r-RESPA integrator ----------------------------
*=======================================================================

*=======================================================================
*==== Phony call to forces: Computes only neighbor lists (OLD UPDATE) --
*=======================================================================

      IF(MOD(nstep,nupdte) .EQ. 0 .AND. lupdate) THEN
         
*=======================================================================
*--  1)  Update shell *H* neighbor list
*=======================================================================
         
         IF(.not.linked_cell) THEN
            CALL mts_forces('u',xpa,ypa,zpa,xpga,ypga,zpga,xpcma,ypcma
     &           ,zpcma,mapnl,mapdn,nmapdn,ucns_p,ucos_p,virs_p,virsp_p
     &           ,ucnp_p,ucop_p,ucnsp_p,ucosp_p,fpx_p,fpy_p,fpz_p
     &           ,stressd_p,nnlppf,nnlpp0)
            rcuth_save=rcuth
            rtolh_save=rtolh
            rcuth=rcutl
            rtolh=0.0D0
         ELSE   
            aux=rcuth+rtolh+rneih 
            IF(cpress .AND. MOD(nstep,nupdte*nupdte_index) .EQ. 0) THEN
               CALL lc_index(indmax,ncx,ncy,ncz,nind,indxi,indxj,indxk
     &              ,aux,co)
               WRITE(kprint,2000)
            END IF
            CALL lc_list(ncx,ncy,ncz,nind,indxi,indxj,indxk,aux,co,xpga
     &           ,ypga,zpga,ngrp,nnlpp0,kprint)
            rcuth_save=rcuth
            rtolh_save=rtolh
            rcuth=rcutl
            rtolh=0.0D0
         END IF 
         
*=======================================================================
*--  2)  Update shell *L* neighbor list
*=======================================================================
         
         CALL mts_forces('h',xpa,ypa,zpa,xpga,ypga,zpga,xpcma,ypcma
     &        ,zpcma,mapnl,mapdn,nmapdn,ucns_p,ucos_p,virs_p,virsp_p
     &        ,ucnp_p,ucop_p,ucnsp_p,ucosp_p,fpx_p,fpy_p,fpz_p,stressd_p
     &        ,nnlpp0,nnlpp)
         rcutl_save=rcutl
         rtoll_save=rtoll
         rcutl=rcutm
         rtoll=0.0D0
         
*=======================================================================
*--  3)  Update shell *M* neighbor list
*=======================================================================
         
         CALL mts_forces('l',xpa,ypa,zpa,xpga,ypga,zpga,xpcma,ypcma
     &        ,zpcma,mapnl,mapdn,nmapdn,ucns_p,ucos_p,virs_p,virsp_p
     &        ,ucnp_p,ucop_p,ucnsp_p,ucosp_p,fpx_p,fpy_p,fpz_p,stressd_p
     &        ,nnlpp,nnlpp1)
         rcuth=rcuth_save
         rtolh=rtolh_save
         rcutl=rcutl_save
         rtoll=rtoll_save
      END IF   
      
*=======================================================================
*---  Advances velocities for half time step TIME using H-forces    ----
*=======================================================================
      
      CALL correc(vpx,vpy,vpz,fpx_h,fpy_h,fpz_h,mass,ntap,time)
      
      IF(cnstpp .NE. 0) THEN
         CALL rattle_correc(time,xp0,yp0,zp0,vpx,vpy
     &        ,vpz,ntap,cnstp,dssp,coeffp,cnstpp,mass,dnit
     &        ,cnst_protp,cnst_protl,mim_lim,iret,errmsg)
         IF(iret .EQ. 1) CALL xerror(errmsg,80,1,2)
      END IF

      CALL correc_stress(cpress,h_skin,nprot,co,oc,vco,vcax,vcay
     &     ,vcaz,fcax_h,fcay_h,fcaz_h,stressd_h,stressr_h,volume
     &     ,press_h,press_kin,pext,tmass,masspp,time,time2)

      IF(isostress) THEN
         CALL rattle_correc_co(co,dssco,cnstco,vco,masspp,iret,errmsg)
         IF(iret .EQ. 1) CALL xerror(errmsg,80,1,2)
      END IF
      
*=======================================================================
*---  Reset h-forces to ZERO                                        ----
*=======================================================================
      
      CALL zeroa(fpx_h,fpy_h,fpz_h,ntap,1)
      tl = time/dfloat(lrespa)
      tl2=tl*0.5D0
      
*=======================================================================
*---     Advances velocities for half time step *time* using        ----
*--- |-->         long range L-forces                               ----
*=======================================================================
      
      DO il=1,lrespa
         
         CALL correc(vpx,vpy,vpz,fpx_l,fpy_l,fpz_l,mass,ntap,tl)
         IF(cnstpp .NE. 0) THEN
            CALL rattle_correc(tl,xp0,yp0,zp0,vpx,vpy,vpz,ntap,cnstp
     &           ,dssp,coeffp,cnstpp,mass,dnit,cnst_protp,cnst_protl
     &           ,mim_lim,iret,errmsg)
            IF(iret .EQ. 1) CALL xerror(errmsg,80,1,2)
         END IF

         CALL correc_stress(cpress,l_skin,nprot,co,oc,vco,vcax,vcay
     &        ,vcaz,fcax_l,fcay_l,fcaz_l,stressd_l,stressr_l,volume
     &        ,press_l,press_kin,pext,tmass,masspp,tl,tl2)

         IF(isostress) THEN
            CALL rattle_correc_co(co,dssco,cnstco,vco,masspp
     &           ,iret,errmsg)
            IF(iret .EQ. 1) CALL xerror(errmsg,80,1,2)
         END IF

*=======================================================================
*---     Reset l-forces to ZERO                                    -----
*=======================================================================
         
         CALL zeroa(fpx_l,fpy_l,fpz_l,ntap,1)
         tm = tl/dfloat(mrespa)
         tm2=tm*0.5D0
         tm4=tm*0.25D0
         
         rshell='m'
         
*=======================================================================
*---        Advances velocities for half time step *time* using     ----
*---    |-->           short range M-forces                         ----
*=======================================================================
         
         DO im=1,mrespa
            ninner=ninner + 1
            nsstt = nsstt + 1
            
            IF(thermos) THEN
c----------  Correct velocities of the NETA thermostats ----------------
c----------  Propagate exp(i L_u tm/4) ---------------------------------
               IF(isostress) THEN
                  CALL rattle_correc_co(co,dssco,cnstco,vco,masspp,iret
     &                 ,errmsg)
                  IF(iret .EQ. 1) CALL xerror(errmsg,80,1,2)
               END IF
               IF(cnstpp .NE. 0) THEN
                  CALL rattle_correc(tm,xp0,yp0,zp0,vpx,vpy,vpz,ntap
     &                 ,cnstp,dssp,coeffp,cnstpp,mass,dnit,cnst_protp
     &                 ,cnst_protl,mim_lim,iret,errmsg)
                  IF(iret .EQ. 1) CALL xerror(errmsg,80,1,2)
               END IF
               CALL comp_thermos_forces(cpress,ndf_thermos,ntap,ss_index
     &              ,nprot,co,vpx,vpy,vpz,vcax,vcay,vcaz,vco,mass,tmass
     &              ,masspr,t,fth)
               CALL correc_etap(neta,etap,fth,qmass,tm4)
            END IF
*----------  Propagate exp(i L_y tm/2) ---------------------------------
            IF(cpress .AND. coupl_mol) THEN
               CALL correc_matr(tm2,co,oc,vco,gmgp,vcax,vcay,vcaz,nprot)
            END IF
            IF(thermos) THEN
c----------  Correct velocities of the NETA thermostats ----------------
c----------  Recompute forces on the thermostat variables --------------
               CALL comp_thermos_forces(cpress,ndf_thermos,ntap,ss_index
     &              ,nprot,co,vpx,vpy,vpz,vcax,vcay,vcaz,vco,mass,tmass
     &              ,masspr,t,fth)
*==========  Propagate exp(i L_u tm/4) =================================
               CALL correc_etap(neta,etap,fth,qmass,tm4)
            END IF
            
            IF(thermos) THEN
*=========  Propagate exp(i L_z tm/4) ==================================
               CALL correc_exp_scale(cpress,slt_exist,slv_exist
     &              ,ss_point(1,1),ss_point(1,2),etap,tm4,nprot,vcax
     &              ,vcay,vcaz,vpx,vpy,vpz,vco)
            END IF

*========= Propagate exp(i L_x tm/2) -==================================

            CALL correc(vpx,vpy,vpz,fpx_m,fpy_m,fpz_m,mass,ntap,tm)

            IF(cnstpp .NE. 0) THEN
               CALL rattle_correc(tm,xp0,yp0,zp0,vpx,vpy,vpz,ntap,cnstp
     &              ,dssp,coeffp,cnstpp,mass,dnit,cnst_protp,cnst_protl
     &              ,mim_lim,iret,errmsg)
               IF(iret .EQ. 1) CALL xerror(errmsg,80,1,2)
            END IF

            CALL correc_stress(cpress,m_skin,nprot,co,oc,vco,vcax,vcay
     &           ,vcaz,fcax_m,fcay_m,fcaz_m,stressd_m,stressr_m,volume
     &           ,press_m,press_kin,pext,tmass,masspp,tm,tm2)
            IF(isostress) THEN
               CALL rattle_correc_co(co,dssco,cnstco,vco,masspp,iret
     &              ,errmsg)
               IF(iret .EQ. 1) CALL xerror(errmsg,80,1,2)
            END IF
            
*=========  Propagate exp(i L_z tm/4) ==================================
            IF(thermos) THEN
               CALL correc_exp_scale(cpress,slt_exist,slv_exist
     &              ,ss_point(1,1),ss_point(1,2),etap,tm4,nprot,vcax
     &              ,vcay,vcaz,vpx,vpy,vpz,vco)
            END IF
            
*=======================================================================
*---        Reset m-forces to ZERO
*=======================================================================
            
            CALL zeroa(fpx_m,fpy_m,fpz_m,ntap,1)
            tn1 = tm/dfloat(n1respa)
            tn12=tn1*0.5D0
            
*=======================================================================
*---                Advance position and velocities for a full      ----
*---       |-->     time step tn1 using intramolecular forces       ----
*=======================================================================
            
            DO in1=1,n1respa
               ninn1=ninn1+1
               CALL correc(vpx,vpy,vpz,fpx_n1,fpy_n1,fpz_n1,mass,ntap
     &              ,tn1)
               IF(cnstpp .NE. 0) THEN
                  CALL rattle_correc(tn1,lx,ly,lz,vpx,vpy,vpz
     &                 ,ntap,cnstp,dssp,coeffp,cnstpp,mass
     &                 ,dnit,cnst_protp,cnst_protl,mim_lim,iret
     &                 ,errmsg)
                  IF(iret .EQ. 1) CALL xerror(errmsg,80,1,2)
               END IF
               CALL correc_stress(cpress,n1_skin,nprot,co,oc,vco,vcax
     &              ,vcay,vcaz,fcax_n1,fcay_n1,fcaz_n1,stressd_n1
     &              ,stressr_n1,volume,press_n1,press_kin,pext,tmass
     &              ,masspp,tn1,tn12)

               IF(isostress) THEN
                  CALL rattle_correc_co(co,dssco,cnstco,vco,masspp,iret
     &                 ,errmsg)
                  IF(iret .EQ. 1) CALL xerror(errmsg,80,1,2)
               END IF

               CALL zeroa(fpx_n1,fpy_n1,fpz_n1,ntap,1)
               tn0 = tn1/dfloat(n0respa)
               tn02=tn0*0.5D0
               
               DO in0=1,n0respa
                  ninn0=ninn0+1
                  
                  CALL dcopy(ntap,lx,1,xpo,1)
                  CALL dcopy(ntap,ly,1,ypo,1)
                  CALL dcopy(ntap,lz,1,zpo,1)

                  if(.not.start_conf)  then
                     CALL verlet(mass,ntap,lx,ly,lz,vpx,vpy,vpz
     &                    ,fpx_n0,fpy_n0,fpz_n0,tn0)
                  else
                     CALL starting_verlet(mass,ntap,lx,ly,lz,vpx,vpy
     &                    ,vpz,fpx_n0,fpy_n0,fpz_n0,tn0,max_dist)
                  endif
                  IF(cnstpp .NE. 0) THEN
                     CALL rattle_verlet(tn0,lx,ly,lz,xpo,ypo,zpo,vpx
     &                    ,vpy,vpz,ntap,cnstp,dssp,coeffp,cnstpp,mass
     &                    ,dnit,cnst_protp,cnst_protl,mim_lim,iret
     &                    ,errmsg)
                     IF(iret .EQ. 1) CALL xerror(errmsg,80,1,2)
                  END IF
                  
                  CALL correc_stress(cpress,n0_skin,nprot,co,oc,vco,vcax
     &                 ,vcay,vcaz,fcax_n0,fcay_n0,fcaz_n0,stressd_n0
     &                 ,stressr_n0,volume,press_n0,press_kin,pext,tmass
     &                 ,masspp,tn0,tn02)
                  IF(isostress) THEN
                     CALL rattle_correc_co(co,dssco,cnstco,vco,masspp
     &                    ,iret,errmsg)
                     IF(iret .EQ. 1) CALL xerror(errmsg,80,1,2)
                  END IF

*----------  Propagate exp(i L_y tm/2) ---------------------------------
                  IF(cpress .AND. coupl_grp) THEN
                     CALL correc_matr(tn02,co,oc,vco,gmgp,vcax,vcay,vcaz
     &                    ,nprot)
                  END IF
 
                  CALL verlet_free(nprot,xpcma,ypcma,zpcma,vcax,vcay
     &                 ,vcaz,tn0)

*---              Reset n-forces to ZERO
                  
                  CALL zeroa(fpx_n0,fpy_n0,fpz_n0,ntap,1)
                  
*=======================================================================
*---           Compute N0-forces                                    ----
*=======================================================================
                  
                  IF(cpress) THEN
                     DO i=1,3
                        DO j=1,3
                           coo(i,j)=co(i,j)
                        END DO
                     END DO
                     
*--- Propagate co matrix for step tm -----------------------------------
                     
                     CALL verlet_free(3,co(1,1),co(1,2),co(1,3),vco(1,1)
     &                    ,vco(1,2),vco(1,3),tn0)
                     IF(isostress) THEN
                        CALL rattle_verlet_co(tm,co,coo,dssco,cnstco,vco
     &                       ,masspp,iret,errmsg)
                        IF(iret .EQ. 1) CALL xerror(errmsg,80,1,2)
                     END IF
                     
                     CALL matinv(3,3,co,oc,volume)
                     volume=volume*boxl**3
                  END IF

*=======================================================================
*---  Recompute coordinates before recomputing forces ------------------
*=======================================================================

                  CALL change_coord_inner(ntap,nprot,protl,ngrp,grppt,co
     &                 ,oc,pmass,lx,ly,lz,xp0,yp0,zp0,xpa,ypa,zpa,xpcm
     &                 ,ypcm,zpcm,xpcma,ypcma,zpcma)

                  CALL mts_intra_n0(xp0,yp0,zp0,xpcma,ypcma,zpcma,fpx_n0
     &                 ,fpy_n0,fpz_n0,ubond,uslvbon,ubend,uslvben,uitors
     &                 ,uslvitor,stressd_n0,tag_bndg)
                  
                  CALL comp_fcm(nprot,protl,fpx_n0,fpy_n0,fpz_n0,fcax_n0
     &                 ,fcay_n0,fcaz_n0,mass,tmass,oc)
                  
                  CALL correc(vpx,vpy,vpz,fpx_n0,fpy_n0,fpz_n0,mass,ntap
     &                 ,tn0)
                  
                  IF(cnstpp .NE. 0) THEN
                     CALL rattle_correc(tn0,lx,ly,lz,vpx,vpy,vpz,ntap
     &                    ,cnstp,dssp,coeffp,cnstpp,mass,dnit,cnst_protp
     &                    ,cnst_protl,mim_lim,iret,errmsg)
                     IF(iret .EQ. 1) CALL xerror(errmsg,80,1,2)
                  END IF
                  
*----------  Propagate exp(i L_y tm/2) ---------------------------------

                  IF(cpress .AND. coupl_grp) THEN
                     CALL correc_matr(tn02,co,oc,vco,gmgp,vcax,vcay,vcaz
     &                    ,nprot)
                  END IF

                  CALL correc_stress(cpress,n0_skin,nprot,co,oc,vco,vcax
     &              ,vcay,vcaz,fcax_n0,fcay_n0,fcaz_n0,stressd_n0
     &              ,stressr_n0,volume,press_n0,press_kin,pext,tmass
     &              ,masspp,tn0,tn02)
               END DO
               CALL mts_intra_n1(xp0,yp0,zp0,xpcma,ypcma,zpcma,fpx_n1
     &              ,fpy_n1,fpz_n1,fudge,lj_fudge,abmd_dir,puhyd
     &              ,conf_bnd_slt_n1,conf_bnd_slv_n1,coul_bnd_slt_n1
     &              ,coul_bnd_slv_n1,unb14,cnb14,ungrp,cngrp,uptors
     &              ,uslvtor,stressd_n1,mapdn,nmapdn,uumb,gr)
               
*=======================================================================
*--- Call force routine for abmd on a torsion angle          -----------
*=======================================================================
               
               CALL comp_fcm(nprot,protl,fpx_n1,fpy_n1,fpz_n1,fcax_n1
     &              ,fcay_n1,fcaz_n1,mass,tmass,oc)
               
               CALL correc(vpx,vpy,vpz,fpx_n1,fpy_n1,fpz_n1,mass,ntap
     &              ,tn1)

               CALL correc_stress(cpress,n1_skin,nprot,co,oc,vco,vcax
     &              ,vcay,vcaz,fcax_n1,fcay_n1,fcaz_n1,stressd_n1
     &              ,stressr_n1,volume,press_n1,press_kin,pext,tmass
     &              ,masspp,tn1,tn12)
            END DO
            
*=======================================================================
*          |-->  END of n-loop                                       ---
*=======================================================================
            
            IF(thermos) CALL verlet_free_eta(neta,eta,etap,tm)
            
*=======================================================================
*---------- recomputes group position and scaled groups ---------------
*=======================================================================
                  
            CALL appbou(xp0,yp0,zp0,xpg,ypg,zpg,pmass,ngrp,grppt)
            
            CALL change_frame(co,oc,-1,ngrp,xpg,ypg,zpg,xpga,ypga
     &           ,zpga)
            
*=======================================================================
*---        Computes M-forces at new coordinates                     ---
*=======================================================================
            
            CALL mts_forces(rshell,xpa,ypa,zpa,xpga,ypga,zpga,xpcma
     &           ,ypcma,zpcma,mapnl,mapdn,nmapdn,ucns_m,ucos_m,virs_m
     &           ,virsp_m,ucnp_m,ucop_m,ucnsp_m,ucosp_m,fpx_m,fpy_m
     &           ,fpz_m,stressd_m,nnlpp1,nnlpp2)
            
            IF(abmd .AND. (associate .OR. dissociate)) CALL
     &           comp_abmd_fdiss(abmd_dir,rspset,diss_list,spring,xpa
     &           ,ypa,zpa,co,fpx_m,fpy_m,fpz_m,uumb,gr)
            
            IF(clewld) THEN
               CALL mts_furier(xp0,yp0,zp0,xpa,ypa,zpa,xpcma,ypcma,zpcma
     &              ,urcsp_m,urcs_m,urcp_m,virsp_m,virs_m,virp_m,fpx_m
     &              ,fpy_m,fpz_m,fsin14,gsin14,fsbend,gsbend,fsbond
     &              ,gsbond,fscnstr_slt,fscnstr_slv,coul_bnd_slt
     &              ,coul_bnd_slv,rshell,rshk,eer_m,stressr_m,fudgec
     &              ,tag_bndg)
            END IF
            
*=======================================================================
*--- Compute force on the c.of.m of each molecule and reduce the -------
*--- atomic force. Do so for   m-shell forces --------------------------
*=======================================================================
            
            CALL comp_fcm(nprot,protl,fpx_m,fpy_m,fpz_m,fcax_m,fcay_m
     &           ,fcaz_m,mass,tmass,oc)
            
*=======================================================================
*---        Scale velocities according to the thermostat velocities ----
*=======================================================================
            
*=========  Propagate exp(i L_z tm/4) ==================================
            IF(thermos) THEN
               CALL correc_exp_scale(cpress,slt_exist,slv_exist
     &              ,ss_point(1,1),ss_point(1,2),etap,tm4,nprot,vcax
     &              ,vcay,vcaz,vpx,vpy,vpz,vco)
            END IF
            
*========= Propagate exp(i L_x tm/2) -==================================

            CALL correc(vpx,vpy,vpz,fpx_m,fpy_m,fpz_m,mass,ntap,tm)

            CALL correc_stress(cpress,m_skin,nprot,co,oc,vco,vcax,vcay
     &           ,vcaz,fcax_m,fcay_m,fcaz_m,stressd_m,stressr_m,volume
     &           ,press_m,press_kin,pext,tmass,masspp,tm,tm2)

*=======================================================================
*---        Scale velocities a second time -----------------------------
*=======================================================================
            
*=========  Propagate exp(i L_z tm/4) ==================================
            IF(thermos) THEN
               CALL correc_exp_scale(cpress,slt_exist,slv_exist
     &              ,ss_point(1,1),ss_point(1,2),etap,tm4,nprot,vcax
     &              ,vcay,vcaz,vpx,vpy,vpz,vco)
            END IF
            
*=======================================================================
*---------  Correct velocities of the NETA thermostats -----------------
*=======================================================================
            
            IF(thermos) THEN
               IF(isostress) THEN
                  CALL rattle_correc_co(co,dssco,cnstco,vco,masspp,iret
     &                 ,errmsg)
                  IF(iret .EQ. 1) CALL xerror(errmsg,80,1,2)
               END IF
               IF(cnstpp .NE. 0) THEN
                  CALL rattle_correc(tm,xp0,yp0,zp0,vpx,vpy,vpz,ntap
     &                 ,cnstp,dssp,coeffp,cnstpp,mass,dnit,cnst_protp
     &                 ,cnst_protl,mim_lim,iret,errmsg)
                  IF(iret .EQ. 1) CALL xerror(errmsg,80,1,2)
               END IF
               CALL comp_thermos_forces(cpress,ndf_thermos,ntap,ss_index
     &              ,nprot,co,vpx,vpy,vpz,vcax,vcay,vcaz,vco,mass,tmass
     &              ,masspr,t,fth)
               CALL correc_etap(neta,etap,fth,qmass,tm4)
            END IF
            
*=======================================================================
*---------  Correct velocities of c.of.m for the barostat --------------
*=======================================================================
            
*==========  Propagate exp(i L_y tm/2) =================================
            IF(cpress .AND. coupl_mol) THEN
               CALL correc_matr(tm2,co,oc,vco,gmgp,vcax,vcay,vcaz,nprot)
            END IF
            
*=======================================================================
*---------  Correct velocities of the NETA thermostats ----------------
*=======================================================================
            
            IF(thermos) THEN
               CALL comp_thermos_forces(cpress,ndf_thermos,ntap,ss_index
     &              ,nprot,co,vpx,vpy,vpz,vcax,vcay,vcaz,vco,mass,tmass
     &              ,masspr,t,fth)
               CALL correc_etap(neta,etap,fth,qmass,tm4)
               CALL comp_thermos_energy(neta,ndf_thermos,t,qmass,eta
     &              ,etap,uceh,hpot,temph)
            END IF

*=======================================================================
*----------  Compute averages and do some analysis at time step M ------
*=======================================================================

            INCLUDE 'mtsmd_avg_inc.f'
            
         END DO
         
*=======================================================================
*---     <--|  END of M-loop                                        ----
*=======================================================================
         
*=======================================================================
*---     Computes L-forces at new coordinates                       ----
*=======================================================================
         
         rshell='l'
         CALL mts_forces(rshell,xpa,ypa,zpa,xpga,ypga,zpga,xpcma,ypcma
     &        ,zpcma,mapnl,mapdn,nmapdn,ucns_l,ucos_l,virs_l,virsp_l
     &        ,ucnp_l,ucop_l,ucnsp_l,ucosp_l,fpx_l,fpy_l,fpz_l,stressd_l
     &        ,nnlpp,nnlpp1)
         
         IF(clewld) THEN 
            CALL mts_furier(xp0,yp0,zp0,xpa,ypa,zpa,xpcma,ypcma,zpcma
     &           ,urcsp_l,urcs_l,urcp_l,virsp_l,virs_l,virp_l,fpx_l
     &           ,fpy_l,fpz_l,fsin14,gsin14,fsbend,gsbend,fsbond,gsbond
     &           ,fscnstr_slt,fscnstr_slv,coul_bnd_slt,coul_bnd_slv
     &           ,rshell,rshk,eer_l,stressr_l,fudgec,tag_bndg)
         END IF
         
*=======================================================================
*--- Compute force on the c.of.m of each molecule and reduce the -------
*--- atomic force. Do so for   l-shell forces --------------------------
*=======================================================================
         
         CALL comp_fcm(nprot,protl,fpx_l,fpy_l,fpz_l,fcax_l,fcay_l
     &        ,fcaz_l,mass,tmass,oc)
         
*---     corrects velocities
         
         CALL correc(vpx,vpy,vpz,fpx_l,fpy_l,fpz_l,mass,ntap,tl)
         
         CALL correc_stress(cpress,l_skin,nprot,co,oc,vco,vcax,vcay
     &        ,vcaz,fcax_l,fcay_l,fcaz_l,stressd_l,stressr_l,volume
     &        ,press_l,press_kin,pext,tmass,masspp,tl,tl2)

      END DO
      
*=======================================================================
*---  <--| END of L-loop                                            ----
*=======================================================================
      
      
*=======================================================================
*---     Computes H-forces (reciprocal +direct lattice)             ----
*---               at new coordinates                               ----
*=======================================================================
      
      rshell='h'
      CALL mts_forces(rshell,xpa,ypa,zpa,xpga,ypga,zpga,xpcma,ypcma
     &     ,zpcma,mapnl,mapdn,nmapdn,ucns_h,ucos_h,virs_h,virsp_h,ucnp_h
     &     ,ucop_h,ucnsp_h,ucosp_h,fpx_h,fpy_h,fpz_h,stressd_h,nnlpp0
     &     ,nnlpp)
      
      IF(clewld) THEN 
         CALL mts_furier(xp0,yp0,zp0,xpa,ypa,zpa,xpcma,ypcma,zpcma
     &        ,urcsp_h,urcs_h,urcp_h,virsp_h,virs_h,virp_h,fpx_h,fpy_h
     &        ,fpz_h,fsin14,gsin14,fsbend,gsbend,fsbond,gsbond
     &        ,fscnstr_slt,fscnstr_slv,coul_bnd_slt,coul_bnd_slv,rshell
     &        ,rshk,eer_h,stressr_h,fudgec,tag_bndg)
      END IF
      
*=======================================================================
*--- Compute force on the c.of.m of each molecule and reduce the -------
*--- atomic force. Do so for   h-shell forces --------------------------
*=======================================================================
      
      CALL comp_fcm(nprot,protl,fpx_h,fpy_h,fpz_h,fcax_h,fcay_h,fcaz_h
     &     ,mass,tmass,oc)
      
*=======================================================================
*---  Corrects velocities                                            ---
*=======================================================================
      
      CALL correc(vpx,vpy,vpz,fpx_h,fpy_h,fpz_h,mass,ntap,time)

      CALL correc_stress(cpress,h_skin,nprot,co,oc,vco,vcax,vcay
     &     ,vcaz,fcax_h,fcay_h,fcaz_h,stressd_h,stressr_h,volume
     &     ,press_h,press_kin,pext,tmass,masspp,time,time2)
      
*=======================================================================
*---  Interaction with a stochastic bath                            ---
*=======================================================================
      
      IF(landersen) THEN
         CALL comp_vel_labframe(vpx,vpy,vpz,vpx,vpy,vpz,co,nprot,protl
     &        ,mass,tmass,vcax,vcay,vcaz)
         CALL collision(ntap,vpx,vpy,vpz,mass,nutime,t,time)
         CALL comp_vcm(vpx,vpy,vpz,oc,nprot,protl,mass,tmass,vcax,vcay
     &        ,vcaz)
      END IF


*=======================================================================
*----------  Dump restart file and do tests at timestep H --------------
*=======================================================================

      IF(debug) THEN
        DO i=1,ntap
          fpx(i)=fpx_n0(i)+fpx_n1(i)+fpx_m(i)+fpx_l(i)+fpx_h(i)
          fpy(i)=fpy_n0(i)+fpy_n1(i)+fpy_m(i)+fpy_l(i)+fpy_h(i)
          fpz(i)=fpz_n0(i)+fpz_n1(i)+fpz_m(i)+fpz_l(i)+fpz_h(i)
        END DO
        fstep=time*DFLOAT(ninner)/dfloat(mrespa*lrespa)
        write(6,764) fstep,fpx_n0(1),fpx_n1(1),fpx_m(1),fpx_l(1)
     &       ,fpx_h(1)
764     FORMAT ('Forces on atom 1 =',f12.2,5e12.5)
c               CALL prtfrc(kprint,ngrp,grppt,nres,M1,prsymb,beta,xp0,yp0
c     &              ,zp0,fpx,fpy,fpz)
      END IF
            
      INCLUDE 'mtsmd_dump_inc.f'

*=======================================================================
*--------         MD LOOP ENDS   !!                    -----------------
*=======================================================================

      IF(nstep .LT. nrject) GOTO 100
      IF(nstep .LT. maxstp) GOTO 100

      IF(annealing) THEN
         grad_max=-1.0D0
         DO i=1,ntap
            fpx(i)=fpx_n0(i)+fpx_n1(i)+fpx_m(i)+fpx_l(i)+fpx_h(i)
            fpy(i)=fpy_n0(i)+fpy_n1(i)+fpy_m(i)+fpy_l(i)+fpy_h(i)
            fpz(i)=fpz_n0(i)+fpz_n1(i)+fpz_m(i)+fpz_l(i)+fpz_h(i)
            IF(DABS(fpx(i)) .GT. grad_max) grad_max=DABS(fpx(i))
            IF(DABS(fpy(i)) .GT. grad_max) grad_max=DABS(fpy(i))
            IF(DABS(fpz(i)) .GT. grad_max) grad_max=DABS(fpz(i))
         END DO
         WRITE(kprint,20000) DABS(grad_max)
         CALL prtfrc(kprint,ngrp,grppt,nres,M1,prsymb,beta,xp0,yp0,zp0
     &        ,fpx,fpy,fpz)
      END IF

*=======================================================================
*     Write timing
*=======================================================================

      CALL timer(vfcp,tfcp,elapse)
      gcpu=-gcpu + tfcp
      elaps= -elaps + elapse
      write(kprint,*)
      write(kprint,60030)
      write(kprint,17000) gcpu
      write(kprint,18000) elaps
      write(kprint,60200) gcpu/time_fs
      write(kprint,60300) elaps/time_fs
      write(kprint,60030)

      IF(wrtgyr) THEN
         CLOSE(kgyr)
      END IF

*================= END OF EXECUTABLE STATEMENTS ========================

1000  FORMAT(////20('*'),'  M o l e c u l a r   T o p o l o g y  ',
     &     20('*')/////)
1100  FORMAT('*',13(' '),
     &     ' I n p u t   O p e r a t i o n s   C o m p l e t e d ',
     &     12(' '),'*'/'*',78(' '),'*')
1110  FORMAT('*',13(' '),
     &     '   M . D.   I n t e r m e d i a t e   R e s u l t s  ',
     &     12(' '),'*'/'*',78(' '),'*')
1200  FORMAT(80('*'))
1300  FORMAT('*',78(' '),'*')
2000  FORMAT(/'<------ Linked cell list indexing updated -------->'/)
13000 FORMAT(/22x,' Temperature has been rescaled ',i5,' times '/)
14000 FORMAT(/22x,'Adjusting bond length to Force Field.'/
     &        22x,'     This will take a while...'//) 
20000 FORMAT(
     &     21x,'***************************************'/
     &     21x,'*        GradMax    ',e12.5,'      *'/
     &     21x,'***************************************')
70200 FORMAT('<------ Dumping Restart File ------->'/)
70300 FORMAT(//'<------ Reading Restart File ------->')
70400 FORMAT('<------ Restart File Read in ------->'//)
70100 FORMAT('Velocities have been rescaled ---->'/)

17000 FORMAT(  15x,' Total cpu time for the run       = ',f10.3)
18000 FORMAT(  15x,' Total elapsed time for the run   = ',f10.3)
60200 FORMAT(  15x,' Averaged time per femtosecond    = ',3x,f7.3)
60300 FORMAT(  15x,' Averaged elapsed per femtosecond = ',3x,f7.3//)
15000 FORMAT(/// 10x,'* * * * r - R E S P A  i s  O N  * * * *'  //)
15500 FORMAT(/   10x,'======= timing ========================='  /)
16011 FORMAT(/5x,'CPUtime for update          =',f10.3)
15011 FORMAT(/5x,'CPUtime for linked cell indexing =',f10.3)
16001 FORMAT(/5x,'CPUtime for h-contribution: RECP =',f8.2,
     &     ' DIR =',f7.3,' TOT =',f7.3)
16002 FORMAT(/5x,'CPUtime for l-contribution: RECP =',f8.2,
     &     ' DIR =',f7.3,' TOT =',f7.3)
16003 FORMAT(/5x,'CPUtime for m-contribution: RECP =',f8.2,
     &     ' DIR =',f7.3,' TOT =',f7.3)
14004 FORMAT(/5x,'CPUtime for n1-contribution  =',f9.4) 
16004 FORMAT(/5x,'CPUtime for n0-contribution  =',f9.4) 
10067 FORMAT(/5x,'THEORIC SPEED UP FOR NON BONDED PART =',f8.2)
10068 FORMAT(/5x,'OVERALL THEORIC SPEED UP =',f8.2/)
80033 format(/5x,'Expected CPU time for the RUN:',I4,
     &     ' hours and ',I2, ' min',/,  
     &       /5x,' Expected average time per M step:',f8.2,' sec.'/  
     &       /5x,' Expected average time per femto :',f8.2,' sec.'/)  
60030 FORMAT(/10x,'==========================================='/)
78410 FORMAT
     &(//' *******ERROR: MAXT for PME too small. INCREASE MAXT.'//)
70700 FORMAT(//15x,' Program Stops Smoothly. Restart Dumped'//)
70120 FORMAT('Velocities of the barostat have been rescaled   ---->'/)
70130 FORMAT('Velocities of the thermostats have been rescaled   ---->'/
     &     )
20900 FORMAT(//' ****** WARNING ! WARNING ! WARNING ***************' /
     &        /' ******   drift_remove ON           ***************' /
     &        /' ****** WARNING ! WARNING ! WARNING ***************' //)
10072 FORMAT(//' --- Total energy remove per particle = ', f12.4,
     &         ' (KJ/mole)'/
     &         ' --- Number of dirty scaling          = ', I10/
     &         ' --- Frequency of scaling             = ',f10.3,
     &         ' 1/ps '/)
80000 FORMAT('REMARK   Rigid body fit on CA atoms')
80100 FORMAT('REMARK   Rigid body fit on heavy atoms')
89000 FORMAT(//
     &     21x,'***************************************'/
     &     21x,'*                                     *'/
     &     21x,'*     Force Field Parameters for      *'/
     &     21x,'*       the current molecules         *'/
     &     21x,'*                                     *'/
     &     21x,'*                                     *'/
     &     21x,'***************************************'//)
89500 FORMAT(//21x,'***************************************'//)
90000 FORMAT('BOND')
91000 FORMAT('BENDING')
92000 FORMAT('TORSION PROPER')
93000 FORMAT('TORSION IMPROPER')
94000 FORMAT('END')
95000 FORMAT('#   Constrained stretchings follow')
96000 FORMAT('NONBONDED MIXRULE')
97000 FORMAT('#   Warning! Mixrules always assumed')
98000 FORMAT(/38('>'),38('<')/
     &     ' Warning number of groups in the restart file is ',
     &     'different from current'/
     &     5x,' Restart group number is ',i7,'  current is  ',i7/38('>')
     &     ,38('<')/)
99000 FORMAT(/38('>'),38('<')/
     &     ' Warning number of molecules in the restart file is ',
     &     'different from current.'/
     &     5x,' Restart molecule number is ',i7,'  current is
     &     ',i7/38('>'),38('<')/)
      RETURN
      END
